@@ -6,6 +6,7 @@
 # under the terms of the MIT License; see LICENSE file for more details.
 
 """Remote application for enabling sign in/up with OpenID Connect."""
+import json
 import traceback
 from typing import Optional
 from urllib.parse import urljoin
@@ -17,6 +18,7 @@ from invenio_db import db
 from invenio_oauthclient.models import RemoteAccount
 from invenio_oauthclient.utils import oauth_link_external_id, \
     oauth_unlink_external_id
+from munch import AutoMunch
 from werkzeug.local import LocalProxy
 
 from .utils import get_dict_from_response
@@ -26,7 +28,7 @@ class LazyOAuthRemoteApp(OAuthRemoteApp):
     """OAuth remote app that can have lazy properties."""
 
     def __init__(self, *args, **kwargs):
-        """constructor."""
+        """Construct new instance."""
         super().__init__(*args, **{
             **kwargs,
             'app_key': 'dummy test'
@@ -55,10 +57,10 @@ class InvenioAuthOpenIdRemote(object):
     name = 'OpenIDC'
     description = 'OpenID Connect'
     icon = ''
-    userinfo_cls = None
+    userinfo_cls = AutoMunch
 
     def remote_app(self) -> dict:
-        """Configures and returns remote app."""
+        """Configure and return remote app."""
         return dict(
             title=self.name,
             description=self.description,
@@ -134,6 +136,12 @@ class InvenioAuthOpenIdRemote(object):
         """Return oauth consumer key."""
         return current_app.config[self.CONFIG_OPENID]['consumer_secret']
 
+    def get_username_fields(self):
+        """Return oauth consumer key."""
+        return current_app.config[self.CONFIG_OPENID].get('username_fields', [
+            'username', 'preferred_username', 'sub', 'email'
+        ])
+
     def get_userinfo(self, remote):
         """Retrieve external user information from the remote userinfo endpoint.
 
@@ -149,10 +157,23 @@ class InvenioAuthOpenIdRemote(object):
 
         response = remote.get(self.get_userinfo_url())
         user_info = self.userinfo_cls(get_dict_from_response(response))
+        user_info.username = self.get_username(user_info)
         session['user_info'] = user_info
 
         return user_info
 
+    def get_username(self, user_info):
+        """Determine username from user info."""
+        for fld in self.get_username_fields():
+            try:
+                return getattr(user_info, fld)
+            except:
+                pass
+        raise Exception(
+            'No username has been found in %s. '
+            'Please specify correct `username_fields` in the configuration' %
+            json.dumps(user_info))
+
     def get_user_id(self, remote, email: str = None) -> str:
         """Determine ID for a given user/email.
 
@@ -213,12 +234,19 @@ class InvenioAuthOpenIdRemote(object):
             oauth_link_external_id(user, {'id': user_id, 'method': self.name})
 
     def handle_authorized(self, resp, remote, *args, **kwargs):
-        """Callback for handling user authorization."""
+        """Handle user authorization.
+
+        :param resp: User authorization response
+        :param remote: The remote application
+        """
         from invenio_oauthclient.handlers import authorized_signup_handler
         return authorized_signup_handler(resp, remote, *args, **kwargs)
 
     def handle_signup(self, remote, *args, **kwargs):
-        """Callback for handling user signup."""
+        """Handle signup.
+
+        :param remote: The remote application
+        """
         from invenio_oauthclient.handlers import signup_handler
         return signup_handler(remote, *args, **kwargs)
 
